<?php
/**
 * @file
 * General helper methods for Availability Calendars, like database access and settings.
 *
 * @author Dan Karran (geodaniel) <dan at karran dot net>
 * @author Nicholas Alipaz (nicholas.alipaz)
 * @author Erwin Derksen (http://drupal.org/user/750928)
 */

/**
 * Utility function to create an array of meta data for the month.
 *
 * @param int $year
 * @param int $month
 * @param object $settings
 * @return array
 */
function availability_calendars_month_meta($year, $month, $settings) {
  $month_meta['daysinmonth'] = date("t", mktime(0, 0, 0, $month, 1, $year));
  $month_meta['firstday'] = date("w", mktime(0, 0, 0, $month, 1, $year)) + $settings->startofweek;
  $temp_days = $month_meta['firstday'] + $month_meta['daysinmonth']; // padding
  $month_meta['weeksinmonth'] = ceil($temp_days / 7);

  // Stop empty weeks occuring at start of month
  if ($month_meta['firstday'] > 6) {
    $month_meta['firstday'] = $month_meta['firstday'] - 7;
    $month_meta['weeksinmonth'] --;
  }
  return $month_meta;
}

/**
 * availability_calendars status options.
 * note: we return unescaped labels as they might be used as options in a select where they get escaped again
 *
 * @return array array with the classes as the keys and the translated but unescaped labels as values
 */
function availability_calendars_options() {
  static $ret = NULL;
  if ($ret === NULL) {
    $ret = array();
    $settings = availability_calendars_get_settings();
    $statuses = availability_calendars_get_states();
    foreach ($statuses as $class => $state) {
      $ret[$class] = $state['label'];
    }
    if ($settings->splitday === 1) {
      foreach ($statuses as $class => $state) {
        $sub = $statuses;
        unset($sub[$class]);
        foreach ($sub as $subclass => $substate) {
          $ret["$class-am $subclass-pm"] = t('!a (am)/!b (pm)', array('!a' => $state['label'], '!b' => $substate['label']));
        }
      }
    }
  }
  return $ret;
}


/**
 * Utility function to get settings related to nodes or administration.
 *
 * @param string $scope type of settings to get: node or system
 * @param int|NULL $arg nid
 * @return object An object with all settings for the given scope
 */
function availability_calendars_get_settings($scope = NULL, $arg = NULL) {
  $settings = new stdClass();
  $settings->pernodeoverride = variable_get('availability_calendars_settings_system_pernodeoverride', 0);

  if ($settings->pernodeoverride && $scope === 'node' && $arg !== NULL) {
    // Add per node settings, NULL if not defined
    $settings->startofweek = variable_get('availability_calendars_settings_node_' . $arg . '_startofweek');
    $settings->showteaser = variable_get('availability_calendars_settings_node_' . $arg . '_showteaser');
    $settings->showkey = variable_get('availability_calendars_settings_node_' . $arg . '_showkey');
    $settings->firstletter = variable_get('availability_calendars_settings_node_' . $arg . '_firstletter');
    $settings->hideold = variable_get('availability_calendars_settings_node_' . $arg . '_hideold');
    $settings->defaultstatus = variable_get('availability_calendars_settings_node_' . $arg . '_defaultstatus');
    $settings->monthcount = variable_get('availability_calendars_settings_node_' . $arg . '_monthcount');
    $settings->splitday = variable_get('availability_calendars_settings_node_' . $arg . '_splitday');
  }

  // Add system wide settings (with their hard coded defaults) as far as not set
  if (!isset($settings->startofweek)) {
    $settings->startofweek = variable_get('availability_calendars_settings_system_startofweek', 1);
  }
  if (!isset($settings->showteaser)) {
    $settings->showteaser = variable_get('availability_calendars_settings_system_showteaser', 1);
  }
  if (!isset($settings->showkey)) {
    $settings->showkey = variable_get('availability_calendars_settings_system_showkey', 1);
  }
  if (!isset($settings->firstletter)) {
    $settings->firstletter = variable_get('availability_calendars_settings_system_firstletter', 0);
  }
  if (!isset($settings->hideold)) {
    $settings->hideold = variable_get('availability_calendars_settings_system_hideold', 0);
  }
  if (!isset($settings->monthcount)) {
    $settings->monthcount = variable_get('availability_calendars_settings_system_monthcount', 12);
  }
  if (!isset($settings->splitday)) {
    $settings->splitday = variable_get('availability_calendars_settings_system_splitday', 0);
  }
  if (!isset($settings->defaultstatus)) {
    $defaultstatus = reset(availability_calendars_get_states());
    $settings->defaultstatus = variable_get('availability_calendars_settings_system_defaultstatus', $defaultstatus['class']);
  }

  // And add the 'system wide only' setting(s)
  $settings->nodeview = variable_get('availability_calendars_settings_system_nodeview', 1);

  return $settings;
}

/*
 * DATABASE ACCESS FUNCTIONS
 * -------------------------
 */

/**
 * Returns an array of records for all states.
 *
 * @return array array of records keyed by the class
 */
function availability_calendars_get_states() {
  static $states = NULL;
  if ($states === NULL) {
    $states = db_select('availability_calendars_states')
      ->fields('availability_calendars_states')
      ->orderBy('weight')
      ->execute()
      ->fetchAllAssoc('class', PDO::FETCH_ASSOC);
  }
  return $states;
}

/**
 * Updates the set of states
 *
 * @param array $states array with the new state records (class, label and weight values)
 */
function availability_calendars_update_states($states) {
  $existing_States = availability_calendars_get_states();
  if ($states != $existing_States) {
    // update states: delete all existing, insert all new states
    db_delete('availability_calendars_states')
      ->execute();
    foreach ($states as $state) {
      db_insert('availability_calendars_states')
        ->fields($state)
        ->execute();
    }
  }
}

/**
 * Returns the notes for the calendar for the given node in the given month.
 * The returned array will be completely filled, so no checking is necessary.
 *
 * @param int $nid
 * @param int $year
 * @param int $month
 * @return array array with 6 week entries of week number (int) => note (string) (possibly empty string)
 */
function availability_calendars_get_node_notes($nid, $year, $month) {
  $notes = db_select('availability_calendars_week')
    ->fields('availability_calendars_week', array('week', 'note'))
    ->condition('nid', $nid)
    ->condition('year', $year)
    ->condition('month', $month)
    ->execute()
    ->fetchAllKeyed();
  // Complete the array with defaults
  $notes += array_fill(1, 6, "");
  return $notes;
}

/**
 * Updates the calendar notes for the given node in the given month.
 *
 * @param int $nid
 * @param int $year
 * @param int $month
 * @param array $notes the (possibly empty) notes keyed by week number
 */
function availability_calendars_update_node_notes($nid, $year, $month, $notes) {
  // Delete all current notes
  db_delete('availability_calendars_week')
      ->condition('nid', $nid)
      ->condition('year', $year)
      ->condition('month', $month)
      ->execute();
  // Insert new notes
  $query = db_insert('availability_calendars_week');
  $values = array('nid' => $nid,'year' => $year,'month' => $month);
  $has_records = FALSE;
  foreach ($notes as $values['week'] => $values['note']) {
    if (!empty($values['note'])) {
      if (!$has_records) {
        $query->fields($values);
        $has_records = TRUE;
      }
      else {
        $query->values($values);
      }
    }
  }
  if ($has_records) {
    $query->execute();
  }
}

/**
 * Returns the states for the calendar for the given node in the given month.
 * The returned array will be completely filled, so no checking is necessary.
 *
 * @param int $nid
 * @param int $year
 * @param int $month
 * @param object $settings settings (containing among others the default status)
 * @return array array with 28 to 31 day states (string) keyed by the day ofg the month number (int).
 */
function availability_calendars_get_node_states($nid, $year, $month, $settings) {
  $states = db_select('availability_calendars_day')
    ->fields('availability_calendars_day', array('day', 'status'))
    ->condition('nid', $nid)
    ->condition('year', $year)
    ->condition('month', $month)
    ->execute()
    ->fetchAllKeyed();
  // Complete the array with defaults
  $days_in_month = date("t", mktime(0, 0, 0, $month, 1, $year));
  $states += array_fill(1, $days_in_month, $settings->defaultstatus);
  return $states;
}

/**
 * Update the states for the calendar for the given node in the given month.
 *
 * @param int $nid
 * @param int $year
 * @param int $month
 * $param array $states array with 28 to 31 day states (string) keyed by the day of the month number (int).
 */
function availability_calendars_update_node_states($nid, $year, $month, $states) {
  // Delete all current states
  db_delete('availability_calendars_day')
      ->condition('nid', $nid)
      ->condition('year', $year)
      ->condition('month', $month)
      ->execute();
  // Insert new states
  $query = db_insert('availability_calendars_day');
  $values = array('nid' => $nid,'year' => $year,'month' => $month);
  $has_records = FALSE;
  foreach ($states as $values['day'] => $values['status']) {
    $values['date'] = "{$values['year']}-{$values['month']}-{$values['day']} 12:00:00";
    if (!$has_records) {
      $query->fields($values);
      $has_records = TRUE;
    }
    else {
      $query->values($values);
    }
  }
  if ($has_records) {
    $query->execute();
  }
}

/**
 * Deletes all calendar information for the given node:
 * - notes
 * - day states
 * - settings
 *
 * @param int $nid
 */
function availability_calendars_delete_node($nid) {
  $nid = (int) $nid;
  db_delete('availability_calendars_week')
      ->condition('nid', $nid)
      ->execute();
  db_delete('availability_calendars_day')
      ->condition('nid', $nid)
      ->execute();
  availability_calendars_delete_node_settings($nid);
}

/**
 * Removes the per node settings for one or all nodes.
 *
 * param int|NULL $nid node id
 */
function availability_calendars_delete_node_settings($nid = NULL) {
  if ($nid === NULL) {
    // Remove per node settings for all nodes
    $nid = '%';
  }
  db_delete('variable')
    ->condition('name', "availability_calendars_settings_node_{$nid}_%", 'LIKE')
    ->execute();
  cache_clear_all('variables', 'cache');
}
